-- In the pubs database, you can run this query to select all 
-- columns and rows from the employee table.
--
SELECT *
FROM employee



-- The following query selects the first name, last name, 
-- and employee ID for each employee from the employee table
--
SELECT fname, lname, emp_id
FROM employee



-- You also can take the previous query and rearrange the 
-- column order from the preceding query. The same information 
-- is returned, but it's displayed in a different column order.
--
SELECT emp_id, lname, fname
FROM employee



-- You can rewrite your query by using the following 
-- SQL Server 2000 syntax statement.
--
SELECT EmployeeID = emp_id, LastName = lname, FirstName = fname
FROM employee


-- You can also rewrite it by using ANSI SQL syntax:
-- 
SELECT emp_id AS EmployeeID,  lname AS LastName,  fname AS FirstName
FROM employee



-- The following query returns first name, last name, 
-- a column containing the literal string Employee ID, 
-- and the employee IDs for all employees from the employee table.
--
SELECT fname, lname, 'Employee ID:', emp_id
FROM employee


-- You can submit the following to return a column called Name, 
-- which is a concatenation of last name, first initial, and employee ID
--
SELECT lname + ', ' + SUBSTRING(fname,1,1) + '.'
AS Name, emp_id as EmployeeID
FROM employee



-- Now look at a more complex query that involves many of the 
-- different pieces you have learned so far:
--
SELECT emp_id AS EmployeeID,  lname + ', ' + SUBSTRING(fname,1,1) +
'.' AS Name, 'Has been employed for ', 
DATEDIFF(year, hire_date, getdate()), ' years.' 
FROM employee



-- The following query uses two system functions to 
-- return the name of the second column of the employee table.
--
SELECT COL_NAME(OBJECT_ID('employee'),2)



-- You can submit the following query to convert the 
-- current date to a character string of eight and a date style of ANSI.
--
SELECT CONVERT(CHAR(8),GETDATE(),2)




-- In this example, you combined the last three columns into 
-- one column by using the CONVERT() function and string concatenation.
--
SELECT emp_id AS EmployeeID,  lname + ', ' + SUBSTRING(fname,1,1) + '.'
AS Name, 'Has been employed for ' + CONVERT(CHAR(2),
(DATEDIFF(year, hire_date, getdate()))) + ' years.'
FROM employee


-- The following query returns the employee ID, 
-- last name, and first name for all employees employed by 
-- the publisher with a pub_id of 0877.
--
SELECT emp_id, lname, fname
FROM employee
WHERE pub_id = '0877'


-- The following query returns the last name and 
-- employee ID for all employees hired between 10/1/92 and 12/31/92.
--
SELECT lname, emp_id
FROM employee
WHERE hire_date BETWEEN '10/1/92' AND '12/31/92'



-- If you want to find employees who work for publishers 
-- with a pub_id of 0877 or 9999, submit the following query.
--
SELECT emp_id, lname, fname
FROM employee
WHERE pub_id IN ('0877', '9999')



-- You also can retrieve rows not in the list by using the NOT 
-- operator. If you want to find all employees who don't work 
-- for publishers with a pub_id of 0877 or 9999, 
-- you can submit the following query.
--
SELECT emp_id, lname, fname
FROM employee
WHERE pub_id NOT IN ('0877', '9999')



-- You can submit the following query to return the title_id 
-- and title of all books with computer anywhere in the title from the titles table.
--
SELECT title_id, title
FROM titles
WHERE  title LIKE '%computer%'



-- You can submit the following query to return 
-- the au_id, au_lname, and au_fname of all authors 
-- whose names begin with B or M from the authors table.  
-- Note that to display names that begin with either 
-- a B, or an M, we used the LIKE clause with both 
-- the B and the M inside of the square brackets.
--
SELECT au_id, au_lname, au_fname
FROM authors
WHERE au_lname LIKE '[BM]%'



-- You can submit this query to find all books that have no sales.
--
SELECT title_id, title
FROM titles
WHERE ytd_sales IS NULL



-- In contrast to the preceding query, you can use the 
-- IS NOT NULL clause to find all books that do have 
-- ytd_sales values by submitting the following query.
--
SELECT title_id, title
FROM titles
WHERE ytd_sales IS NOT NULL



-- Consider the following: You want to retrieve the title ID, 
-- title, and price for all books that have a pub_ ID of 
-- 0877 or the word computer in the title, and for 
-- which the price is NOT NULL.
--
SELECT title_id, title, price, pub_id
FROM titles
WHERE title LIKE '%computer%' OR pub_id  = '0877' AND price IS NOT NULL



-- Now run the query again and see whether you can get rid of 
-- that NULL value in your price field.
--
SELECT title_id, title, price, pub_id
FROM titles
WHERE (title LIKE '%computer%' OR pub_id  = '0877') AND price IS NOT NULL



-- These queries return the same ordered result sets.
-- Query1
--
SELECT title_id, au_id
FROM titleauthor
ORDER BY title_id, au_id

--Query2
--
SELECT title_id, au_id
FROM titleauthor
ORDER BY 1, 2



-- For example, this query returns a count of the total 
-- number of rows in the employee table.
--
SELECT COUNT(*)
FROM employee



-- This query selects the maximum value found in the 
-- ytd_sales column from the titles table.
--
SELECT MAX(ytd_sales)
FROM titles



-- This query selects all the values in the qty column 
-- of the sales table and adds them.
--
SELECT SUM(qty)
FROM sales



-- The HAVING clause has the same effect on the 
-- GROUP BY clause as the WHERE clause has on 
-- the SELECT statement. For example, the following 
-- query finds all books that have more than one author 
-- and returns the title_id and number of authors for each book.
--
SELECT title_id, count(title_id) AS Number_of_Authors
FROM titleauthor
GROUP BY title_id
HAVING count(title_id) > 1



-- If you want to find all books with year-to-date sales of 
-- $4,000 or more and list all title IDs, you can submit the following query.
--
SELECT title_id, ytd_sales
FROM titles
WHERE (ytd_sales>=4000)
GROUP BY ALL title_id, ytd_sales



-- The following examples compare the COMPUTE BY and GROUP BY clauses.
--
SELECT type, SUM(ytd_sales)
FROM titles
GROUP BY type



-- The order of the columns in COMPUTE BY must be the 
-- same as those in ORDER BY, and you can't skip columns. 
-- Now enter the following and look at the results.
--
SELECT type, ytd_sales
FROM titles
ORDER BY type
COMPUTE SUM(ytd_sales) BY type



-- The ROLLUP operator is typically used to produce running 
-- averages or running sums. You do so by applying the aggregate 
-- function in the SELECT column_list to each column in the 
-- GROUP BY clause moving from left to right. 
--
SELECT type, pub_id,
SUM(ytd_sales) AS ytd_sales
FROM titles
GROUP BY type, pub_id
WITH ROLLUP



-- The CUBE operator produces super-aggregate rows by using 
-- every possible combination of the columns in the GROUP BY 
-- clause. Like the ROLLUP operator, the CUBE operator produces 
-- the running averages and running sums but also cross-references 
-- columns to return additional summary rows. 
--
SELECT type, pub_id,
SUM(ytd_sales) AS ytd_sales
FROM titles
GROUP BY type, pub_id
WITH CUBE



-- In this example, the SELECT statement selects all columns 
-- from the publishers and pub_info tables when the pub_id 
-- columns for the joined tables are equal. 
-- Notice the redundant pub_id column.
--
--Older ANSI Statement
--
SELECT pub_id, pub_name, pr_info
FROM publishers, pub_info
WHERE publishers.pub_id = pub_info.pub_id

--The Preferred ANSI Statement
--
SELECT pub_id, pub_name, pr_info
FROM publishers
INNER JOIN pub_info ON publishers.pub_id = pub_info.pub_id



-- In a natural join, column values are compared for equality, 
-- but redundant columns are eliminated from the columns in 
-- the result set. In the following example, the SELECT statement 
-- selects all columns from the publishers table and all columns 
-- except pub_id from the pub_info table.
-- Older ANSI Statement
--
SELECT publishers.*, pub_info.logo, pub_info.pr_info
FROM publishers, pub_info
WHERE publishers.pub_id = pub_info.pub_id

-- Preferred ANSI Statement
--
SELECT publishers.*, pub_info.logo, pub_info.pr_info
FROM publishers
INNER JOIN pub_info ON publishers.pub_id = pub_info.pub_id



-- As an example of creating a Cartesian product by using the 
-- cross join or an unrestricted join, say that you want to list all 
-- book titles and their authors' IDs. To do so, you submit the following query.
-- Older ANSI Syntax
--
SELECT titles.title, titleauthor.au_id
FROM titles, titleauthor

-- Preferred ANSI Syntax
--
SELECT titles.title, titleauthor.au_id
FROM titles CROSS JOIN titleauthor



-- If you want to find all the titles, whether they have sold any copies, 
-- and the number of copies sold, you can submit the following 
-- query using the newer SQL '99 syntax.
--
SELECT titles.title_id, titles.title, sales.qty
FROM titles LEFT OUTER JOIN sales
ON titles.title_id = sales.title_id



-- If you want to list all authors who live in the same city and ZIP code, 
-- you compare city and ZIP by executing the following query.
-- Older ANSI Syntax
--
SELECT au1.au_fname, au1.au_lname,
au2.au_fname, au2.au_lname,
au1.city, au1.zip
FROM authors au1, authors au2
WHERE au1.city = au2.city
AND au1.zip = au2.zip
AND au1.au_id < au2.au_id
ORDER BY au1.city, au1.zip

-- Preferred ANSI Syntax
--
SELECT au1.au_fname, au1.au_lname,
au2.au_fname, au2.au_lname,
au1.city, au1.zip
FROM authors au1
INNER JOIN authors au2 ON au1.city = au2.city
AND au1.zip = au2.zip
WHERE au1.au_id < au2.au_id
ORDER BY au1.city, au1.zip



-- To find all authors who live in the same state as the bookstore 
-- that sells their publishers' books, you can run either of 
-- the following queries.
--
SELECT DISTINCT au_fname, au_lname, state
FROM authors
WHERE state IN
(SELECT state FROM stores)


SELECT DISTINCT au_fname, au_lname, state
FROM authors
WHERE EXISTS
(SELECT * FROM stores
WHERE state = authors.state)



-- In the following example, the join query is rewritten as a correlated subquery. 
-- The queries return the same information. The queries answer the following 
-- instruction: Show me authors who live in the same city and ZIP code.
-- Using a JOIN
--
SELECT au1.au_fname, au1.au_lname,
au2.au_fname, au2.au_lname,
au1.city, au1.zip
FROM authors au1, authors au2
WHERE au1.city = au2.city
AND au1.zip = au2.zip
AND au1.au_id < au2.au_id
ORDER BY au1.city, au1.zip

-- Using a Correlated Subquery
--
SELECT au1.au_fname, au1.au_lname, au1.city, au1.zip
FROM authors au1
WHERE zip IN
(SELECT zip
FROM authors au2
WHERE au1.city = au2.city
AND au1.au_id <> au2.au_id)
ORDER BY au1.city, au1.zip



-- I have created a temporary table called #tmpTitles with a list of title ID's, 
-- the title itself and it's selling price:
--
SELECT title_id, title, price
INTO #tmpTitles
FROM titles
GO
SELECT * FROM #tmpTitles
GO



-- In this example, you can use the the UNION operator to pull 
-- data from two separate queries and combine the two result 
-- sets into a single result set:
--
SELECT title, stor_name, ord_date, qty
FROM titles, sales, stores
WHERE titles.title_id = sales.title_id
AND stores.stor_id = sales.stor_id
UNION
SELECT title, 'No Sales', NULL, NULL
FROM titles
WHERE title_id NOT IN
(SELECT title_id FROM sales)
ORDER BY qty



