-- Before going any further, take a minute to copy some of your 
-- data to backup tables so that later you can refresh the tables 
-- you are working with. Run the following queries to create copies 
-- of your publishers and sales tables. Be sure to turn on the 
-- Select Into/Bulk Copy database option before you attempt 
-- to run these statements.
--
USE pubs
GO
EXEC sp_dboption 'pubs', 'Select into/bulkcopy', TRUE
GO
SELECT * INTO tmpPublishers FROM publishers
SELECT * INTO tmpStores FROM stores
SELECT * INTO tmpTitles FROM titles
SELECT * INTO tmpSales FROM sales
GO
EXEC sp_dboption 'pubs', 'Select into/bulkcopy', FALSE
GO



-- If you don't have SQL Server Enterprise Manager readily available, 
-- you can execute the sp_help system stored procedure to get the 
-- same information.  (Database information.)
--
USE pubs
go
EXEC sp_help publishers
go



-- You can also use the INFORMATION_SCHEMA views to 
-- gather information. For example, to find out the column names 
-- and data types in the preceding example, you could run the following query.
--
USE pubs
GO
SELECT * FROM INFORMATION_SCHEMA.Columns
WHERE TABLE_NAME = 'publishers'
GO



-- To insert a single row into the tmpPublishers table, 
-- you can execute the following command.
--
USE pubs
GO
INSERT INTO tmpPublishers
VALUES('9956', 'A New Publisher', 'Poulsbo', 'WA', 'USA')



-- The following INSERT statement has a list of column names 
-- before the VALUES clause, and that list includes only a subset 
-- of the column names in the table. The values list then needs to 
-- have values only for the columns listed.
--
INSERT INTO tmpPublishers(state, pub_id)
VALUES('AK', '9932')



-- After executing the INSERT statement, you can run the following query 
-- to see the row you have just inserted:
--
SELECT * FROM tmpPublishers
WHERE pub_id = '9932'



-- This INSERT doesn't supply a value for the pub_id column:
--
INSERT INTO tmpPublishers(pub_name, city, state)
VALUES('The Best Books', 'New Orleans', 'LA')



-- You can use the keyword DEFAULT in the actual values 
-- list as a way of telling SQL Server that it should determine 
-- what value should be used. 
--
INSERT INTO tmpPublishers
VALUES('9950', DEFAULT, DEFAULT,'AK', DEFAULT)



-- You can simply tell SQL Server to use all default values 
-- by using the keywords DEFAULT VALUES, as in the 
-- following INSERT statement.
--
INSERT INTO tmpPublishers DEFAULT VALUES



-- The number of columns in the subquery's result set must 
-- match the number of columns in the table, and the columns 
-- must have compatible data types. In this first example, 
-- you will create a table to keep track of addresses.
--
CREATE TABLE tblAddressList
(strName varchar(50) not null,
strAddress varchar(50) not null,
strCity varchar(20) not null,
strState char(2)  )



-- This table has four character fields, so any SELECT statement 
-- you use to populate it must return four character columns. 
--
INSERT INTO tblAddressList
SELECT stor_name, stor_address, city, state
FROM tmpStores



-- The address_list table is expecting just a single value 
-- for its name column, so you can concatenate the last and 
-- first names together. You can include a comma and space 
-- in the concatenated result.
--
INSERT INTO tblAddressList
SELECT au_lname + ', ' + au_fname, address, city, state
FROM authors



-- This last example creates a table to keep track of the names 
-- of all the publishers and the titles of all the books each 
-- publisher has published:
--
CREATE TABLE tblPublisherList
(strPubName varchar(40) NULL,
strTitle varchar(80) NULL)



-- To populate this table, you need to join the 
-- tmpPublishers and tmpTitles tables. Use an outer join 
-- so that those publishers that don't currently have any books 
-- published are included.
--
INSERT INTO tblPublisherList
SELECT pub_name, title
FROM tmpPublishers LEFT OUTER JOIN tmpTitles
ON tmpPublishers.pub_id = tmpTitles.pub_id



-- Look at the output of the sp_spaceused system stored procedure 
-- (your results might vary slightly):
--
EXEC sp_spaceused tmpPublishers



-- The following table should be able to hold the results:
--
CREATE TABLE tblSpaceUsage
(strTableName varchar(30) not null,
strRows varchar(9),
strReserved varchar(10),
strData varchar(10),
strIndexSize varchar(10),
strUnused varchar(10)  )



-- To insert into this table, you can execute the 
-- system stored procedure sp_spaceused:
--
INSERT INTO tblSpaceUsage
EXEC sp_spaceused 'tmpPublishers'



-- The following DELETE statement removes all rows from the tmpSales table:
--
DELETE tmpSales


-- The following DELETE statement removes all books with 
-- a pub_name of "New Moon Books" from the tmpPublishers table.
--
DELETE tmpPublishers
WHERE pub_name = 'New Moon Books'



-- In this next example, you'll remove all titles published by New Moon Books. 
-- The tmpTitles table is the one to be modified, but it contains only the 
-- publisher ID, not the publisher name. You must look in the publishers 
-- table to find the publisher ID for New Moon Books 
-- (we already deleted the ID from the tmpPublishers table), which then 
-- determines the rows to remove from the tmpTitles table.
--
DELETE FROM tblPublisherList
WHERE strPubName =
(SELECT pub_name FROM tmpPublishers
WHERE pub_id = '9956')



-- The preceding query could have been written more efficiently as a JOIN. 
-- When you use a JOIN with a DELETE command, you specify two FROM 
-- keywords: one to specify the table to delete records from and one to specify 
-- the join table. This example performs the same delete shown earlier.
--
DELETE tblPublisherList
FROM tblPublisherList pl JOIN tmpPublishers p
ON pl.strPubName = p.pub_name
AND p.pub_id = '9956'



-- The next example uses a lookup table that returns more 
-- than one value. You'll remove all rows from the tmpSales 
-- table that indicate the sale of business books. The tmpSales 
-- table holds the title_id value of the book sold, but not its type. 
-- You must access the tmpTitles table to find which title_ids 
-- correspond to business books. Because you deleted the information 
-- in the tmpSales table earlier today, you need to repopulate the table. 
-- Remember that when you started this lesson, you copied all the sales 
-- information to another table. Now drop the tmpSales table, and 
-- then re-create and repopulate it by using a SELECT INTO statement.
--
USE pubs
GO
DROP TABLE tmpSales
GO
SELECT * INTO tmpSales
FROM Sales
GO

-- Now remove all rows from the tmpSales table that are of type "business":
--
DELETE tmpSales
WHERE title_id IN
(SELECT title_id FROM tmpTitles
WHERE type = 'business')



-- The following example shows how the DELETE statements 
-- use multiple tables in a FROM clause:
--
DELETE tmpSales
FROM tmpSales s JOIN tmpTitles t
ON s.title_id = t.title_id
WHERE t.type = 'business'



-- If you really want to remove all data from a table while leaving the 
-- table structure intact, you can resort to another alternative.
--
TRUNCATE TABLE tmpSales



-- Before you go any further, now is a good time to make sure 
-- that you have fresh data. Run the following queries to refresh 
-- the tmpSales, tmpPublishers, tmpStores, and tmpTitles tables:
--
USE pubs
GO
DROP TABLE tmpSales
DROP TABLE tmpPublishers
DROP TABLE tmpStores
DROP TABLE tmpTitles
GO
SELECT * INTO tmpPublishers FROM publishers
SELECT * INTO tmpSales FROM sales
SELECT * INTO tmpStores FROM stores
SELECT * INTO tmpTitles FROM titles
GO



-- The following UPDATE statement changes the ytd_sales column 
-- in the tmpTitles table to 0 for every row. You might want to update 
-- this column at the beginning of every year.
--
UPDATE tmpTitles
SET ytd_sales = 0



-- The following example updates the city column for the 
-- publisher Algodata Infosystems:
--
UPDATE tmpPublishers
SET city = 'El Cerrito'
WHERE pub_name = 'Algodata Infosystems'



-- An UPDATE statement can make the new value in the column 
-- dependent on the original value. The following example 
-- changes the price of all psychology books to 10 percent 
-- less than the current price:
--
UPDATE tmpTitles
SET price = price * 0.90
WHERE type = 'psychology'



-- An UPDATE statement can change more than one column. 
-- The word SET occurs only once, and the different columns 
-- to be changed are separated by commas. The following 
-- UPDATE statement increases the price of all popular computing 
-- books by 20 percent and appends the string (price increase) to the 
-- notes field of the same rows:
--
UPDATE tmpTitles
SET price = price * 1.2, notes = notes + ' (price increase)'
WHERE type = 'popular_comp'



-- A single UPDATE statement can change rows from a single table only. 
-- However, SQL Server allows you to include another table in your 
-- UPDATE statement to be used as a lookup table. Usually, the 
-- lookup table appears in a subquery, which can appear in either 
-- the WHERE clause or the SET clause of the UPDATE statement. 
-- In this next example, you'll change the publisher of all business books 
-- to New Moon Books:
--
UPDATE tmpTitles
SET pub_id =
(SELECT pub_id FROM tmpPublishers
WHERE pub_name = 'New Moon Books')
WHERE type = 'business'



-- The following example shows how you can rewrite the previous 
-- UPDATE statement by using multiple tables in a FROM clause:
--
UPDATE tmpTitles
SET tmpTitles.pub_id = p.pub_id
FROM tmpTitles t JOIN tmpPublishers p
ON t.pub_id = p.pub_id
WHERE p.pub_name = 'New Moon Books'
AND type = 'business'



-- This example changes the publisher of all psychology books 
-- published by New Moon Books to Binnet & Hardley:
--
UPDATE tmpTitles
SET pub_id =
    (SELECT pub_id FROM tmpPublishers
     WHERE pub_name = 'Binnet & Hardley')
WHERE type = 'psychology' AND pub_id =
    (SELECT pub_id FROM tmpPublishers
     WHERE pub_name = 'New Moon Books')



